/*      
        alph: Piper program implementing and analysing traditional and
        historic c(ai)phers designed to be used in conjuction with other
        programs.

        Copyright (C) 2004  Corcalciuc V. Horia

        This library is free software; you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public
        License as published by the Free Software Foundation; either
        version 2.1 of the License, or (at your option) any later version.

        This library is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
        Lesser General Public License for more details.

        You should have received a copy of the GNU Lesser General Public
        License along with this library; if not, write to the Free Software
        Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

        Dedicated to Rudolf.
*/


/* System */
#include <stdio.h>
#include <string.h>

/* Our own. */
#include "alphCRYPT.h"
#include "alphan.h"
#include "config.h"

#define MODERN 12
#define HASHES 15

void 
usage ( char *copts[], char *copts_desc[], char *tol, char *tol_desc[], char tgg[], char *tgg_desc[] ) 
{
  int forw;

  printf ("\n\t\t%s v%s\n", PACKAGE_NAME, PACKAGE_VERSION);
  printf ("\n Classic:\n");
  for (forw=0; forw < 22; forw++)
    {
      switch(forw)
        {
          case MODERN:
            printf ("\n Modern:\n");
            break;
          case HASHES:
            printf ("\n Hashes: \n");
            break;
        }
      if(forw==6 || forw==7)
        printf("\t%s\xB9 \t%s\n", copts[forw], copts_desc[forw]);
      else
        printf ("\t%s \t%s\n", copts[forw], copts_desc[forw]);
    }
  printf("\n Tools:\n");
  for(forw=0; forw <4; forw++)
    {
      if(forw==3)
        printf("\t--plug=<SUB>\t%s\n", "<SUB> is the plugboard alphabet sequence");
      else
        printf ("\t-%c \t\t%s\n", tol[forw], tol_desc[forw]);
    }
  printf("\n Options:\n");
  for (forw=0; forw < 2; forw++)
    if (forw!=2)
      printf ("\t-%c \t\t%s\n", tgg[forw], tgg_desc[forw]);
    else if(forw==2)
      printf("\t-%c n \t\t%s\n", tgg[forw], tgg_desc[forw]);
  printf("\t-h, --help\t%s\n", "this usage info");
  printf ("\n\t\xB9 \bComplex parameters, please refer to manual for usage details.\n\n");
  printf ("\t\t%c Corcalciuc V. Horia - rain@sevenrains.ro\n\n", 169);
  
  exit(1);
}

int main(argc, argv)
int argc;
char **argv;
{

  int topt=argc;
  int optr;
  int forw;
  int xtime;
  char *key[5];
  int *numkey;

  char *copts[] = { "--CAESAR",
                    "--ATBASH",  
                    "--VIGENERE",
                    "--VERNAM",
                    "--PLAYFAIR",
                    "--BIFID",
                    "--PURPLE",
                    "--ENIGMA",
                    "--TWOSQUARE",
                    "--LEWIS",
                    "--ALBERTI",
                    "--ADFGX",
                    "--MORSE",
                    "--ROT13",
                    "--FEISTEL",
                    "--BASE64",
                    "--UNIXELF",
                    "--WEINBERGER",
                    "--SDBM",
                    "--DJB2",
                    "--XOR",
                    "--MD5"
                  };

  char cid[] = { 'c', 'a', 'v', 'u', 'p', 'b', 'y', 'e', 't', 'l', 'i', 'x', 'm', 'r', 'f', '6', 'w', 'g', 's', '2', 'O', '5' };
  char *copts_desc[] = { "",
                         "",
                         "KEY",
                         "FILE",
                         "KEY",
                         "KEY",
                         "SIXES SW1 SW2 SW3 MOTION",
                         "ROTORS REFLECTOR RING",
                         "KEY KEY",
                         "KEY",
                         "KEY",
                         "",
                         "",
                         "",
                         "KEY",
                         "",
                         "",
                         "",
                         "",
                         "",
                         "\tLENGTH",
                         ""
                       };
  char tgg[] = { 'e', 'd'};
  char *tgg_desc[] = { "encrypt the stream (def.)",
                       "decrypt the stream",
                     };
  char tol[] = { 's', 'p', 'c' };
  char *tol_desc[] = { "frequency attack [n] times (def: 1)",
							  "permutations of <STRING>",
                "combinations of <STRING> in blocks of <n>"
							 };
  typedef enum { false = 0, true } flip;
  flip dsmash = false, noeot = false, plugs=false;
  struct input {
    char *pdata;
    unsigned long hdata;
    char plug[26];
  } *data;
  int *ints;
  int incr;
  FILE *fp;


  for (optr=argc-1; optr >= 0; optr--)
    if (strlen (argv[optr]) < 7 || argc==1)
      if (argc==1 || strstr(argv[optr], "-h") || strstr(argv[optr], "--help") || strstr(argv[optr], "/?"))
        {
            usage(copts, copts_desc, tol, tol_desc, tgg, tgg_desc);
            return 1;
        }
      else if( strstr(argv[optr], "-p") || strstr(argv[optr], "-c") )
        noeot=true;

  data = (struct input *) calloc(1, sizeof(struct input));
  data->pdata = (char *) calloc (BUFSIZ, sizeof(char));

  if(!noeot)
    {
      fflush(stdin);
      for (optr = 0, incr=1; optr <= BUFSIZ; optr++)
        {
          if(optr == BUFSIZ && !feof(stdin))
            data->pdata = (char *) realloc(data->pdata, (++incr)*BUFSIZ);
          if (!feof (stdin))
            data->pdata[optr] = fgetc (stdin);
          else
            {
              data->pdata[optr -1] = '\0';
              break;
            }
        }
    }

  for(optr=0; optr<argc; optr++)
    if (strstr(argv[optr], "plug") && plugs==false)
      {
        plugs=true;
        if (argv[optr][0]=='-' && argv[optr][1]=='-')
          for(forw=0, incr=0, xtime=0; forw<33; forw++)
            {
              if(argv[optr][forw]=='=')
                incr++;
              if(incr && argv[optr][forw]!='=')
                data->plug[xtime++]=toupper(argv[optr][forw]);
             }
        if(strlen(data->plug)!=26)
          return 1;
        for(forw=0; forw<strlen(data->pdata); forw++)
          if(isalpha(data->pdata[forw]))
            {
              xtime=tolower(data->pdata[forw])-97;
              xtime=tolower(data->plug[xtime]);
              if(isupper(data->pdata[forw]))
                data->pdata[forw]=xtime-32;
              else
                data->pdata[forw]=xtime;
            }
      }

  while (argc-->0)
    {
      for (optr=0; optr<22; optr++)
        {
          if (!strcmp(copts[optr],argv[argc]-'\0'))
            switch (cid[optr])
              {
                case 'c':
                  data->pdata=caesar (data->pdata);
                  break;
                case 'a':
                  data->pdata=atbash (data->pdata);
                  break;
                case 'r':
                  data->pdata=rot13 (data->pdata);
                  break;
                case 'v':
                  if (argv[argc+1]==NULL)
                    return 1;
                  key[0] = (char *) calloc (strlen(argv[argc+1]), sizeof(char));
                  strcpy (key[0], argv[argc+1]);
                  data->pdata=vigenere (data->pdata, key[0]);
                  break;
                case 'u':
                  if (argv[argc+1]==NULL)
                    return 1;
                  incr = 0;
                  key[0] = (char *) calloc (strlen(data->pdata), sizeof(char));
                  if ((fp = fopen (argv[argc+1], "r")) == NULL)
                    exit(1);
                  else
                    while ((forw = fgetc(fp)) != EOF && incr < strlen(data->pdata))
                      if (isalpha (forw))
                        key[0][incr++] = forw;
                  data->pdata=vernam (data->pdata, key[0]);
                  break;
                case 'p':
                  if (argv[argc+1]==NULL)
                    return 1;
                  key[0] = (char *) calloc (strlen(argv[argc+1]), sizeof(char));
                  for (incr=0; incr <= strlen(argv[argc+1]); incr++)
                    key[0][incr]=argv[argc+1][incr];
                  data->pdata=playfair (data->pdata, key[0]);
                  break;
                case 'b':
                  if (argv[argc+1]==NULL)
                    return 1;
                  if ( argc+1 > topt || !isalpha(argv[argc+1][0]))
                    return 1;
                  key[0] = (char *) calloc (strlen(argv[argc+1]), sizeof(char));
                  for (incr=0; incr <= strlen(argv[argc+1]); incr++)
                    key[0][incr]=argv[argc+1][incr];
                  data->pdata=bifid (data->pdata, key[0]);
                  break;
                case 'y':
                  for (forw=1; forw<6; forw++)
                  {
                    if (argc+forw > topt)
                      return 1;
                    if ( forw==1 || forw==2 || forw==3 || forw==4 )
                      {
                        if ( !isdigit(argv[argc+forw][0]) )
                          return 1;
                      }
                    key[forw-1] = (char *) calloc(strlen(argv[argc+forw]), sizeof(char));
                    strcpy ( key[forw-1], argv[argc+forw] );
                  }
                  data->pdata=purple ( data->pdata, key[0], key[1], key[2], key[3], key[4] );
                  break;
                case 'e':
                  for (forw=1; forw<4; forw++)
                  {
                    if ( argc+forw > topt )
                        return 1;
                    if ( forw==1 )
                      {
                        if ( !isdigit(argv[argc+forw][0]) )
                          return 1;
                      }
                    else
                      {
			                  if ( !isalpha(argv[argc+forw][0]) )
                          return 1;
                      }
			
                    key[forw-1] = (char *) calloc (strlen(argv[argc+forw]), sizeof(char));
                    strcpy ( key[forw-1], argv[argc+forw] );
                  }
                  data->pdata=enigma ( data->pdata, key[0], key[1], key[2] );
                  break;
                case 'm':
                  data->pdata=morse(data->pdata);
                  break;
                case 't':
                  for (forw=1; forw<3; forw++)
                  {
                    if ( argc+forw > topt )
                        return 1;
			            if ( !isalpha(argv[argc+forw][0]) )
                          return 1;
                    key[forw-1] = (char *) calloc (strlen(argv[argc+forw]), sizeof(char));
                    strcpy ( key[forw-1], argv[argc+forw] );
                  }
                  data->pdata=twosquare ( data->pdata, key[0], key[1] );
                  break;
                case 'l':
                  if (argv[argc+1]==NULL)
                    return 1;
                  key[0] = (char *) calloc (strlen(argv[argc+1]), sizeof(char));
                  for (incr=0; incr <= strlen(argv[argc+1]); incr++)
                    key[0][incr]=argv[argc+1][incr];
                  data->pdata=lewis (data->pdata, key[0]);
                  break;
                case 'i':
                  if (argv[argc+1]==NULL)
                    return 1;
                  key[0] = (char *) calloc (strlen(argv[argc+1]), sizeof(char));
                  for (incr=0; incr <= strlen(argv[argc+1]); incr++)
                    key[0][incr]=argv[argc+1][incr];
                  data->pdata=alberti (data->pdata, key[0]);
                  break;
                case 'x':
                  data->pdata=adfgx(data->pdata);
                  break;
                case 'f':
                  if (argv[argc+1]==NULL)
                    return 1;
                  key[0] = (char *) calloc (strlen(argv[argc+1]), sizeof(char));
                  for (incr=0; incr <= strlen(argv[argc+1]); incr++)
                    key[0][incr]=argv[argc+1][incr];
                  data->pdata=feistel (data->pdata, key[0]);
                  break;
                case '6':
                  numkey = (int *) calloc(strlen(data->pdata), sizeof(int));
                  for (incr=0, forw=0; incr < strlen(data->pdata); incr++)
                    numkey[forw++]=(int) data->pdata[incr];
						      data->pdata=base64(numkey);
                  break;
                case 'w':
                  data->hdata=unixhash(data->pdata);
                  break;
                case 'g':
                  data->hdata=weinberger(data->pdata);
                  break;
                case 's':
                  data->hdata=sdbmh(data->pdata);
                  break;
                case '2':
                  data->hdata=djb2(data->pdata);
                  break;
                case 'O':
                  if (argv[argc+1]==NULL)
                    return 1;
                  forw=0;
                  forw+=atoi(argv[argc+1]);
                  data->hdata=XORhash(data->pdata, forw);
                  break;
                case '5':
                  data->pdata=md5hash(data->pdata);
                  break;
              }
          else if (argv[argc][1]==tgg[optr])
			     {
            if (argv[argc][0]=='-')
                  switch (tgg[optr])
                  {
                    case 'e':
                      al_dec=false;
                      break;
                    case 'd':
                      al_dec=true;
                      break;
                  }
            }
			    else if (argv[argc][1]==tol[optr])
             if (argv[argc][0]=='-')
               switch (tol[optr])
                 {
                   case 's':
                     if (strchr(data->pdata, ' '))
                       {
                         dsmash=true;
                         if(argv[argc+1]!=NULL)
                           {
                             xtime=atoi(argv[argc+1]);
                             if (xtime<0 || xtime>9)
                               return 2;
                           }
                         else
                             xtime=true;
                       }
                     else
                       return 2;
                     break;
						       case 'p':
                     if(argv[argc+1]!=NULL)
							         {
                         permute(argv[argc+1]);
                         exit(0);
                       }
                     else
                       return 2;
                     break;
                   case 'c':
                     if(argv[argc+1]!=NULL && atoi(argv[argc+2])<=strlen(argv[argc+1]))
                       {
                         combine(argv[argc+1], atoi(argv[argc+2]));
                         exit(0);
                       }
                     else
                       return 2;
                     break;
                 }
        }
    }

  if(plugs)
    for(forw=0; forw<strlen(data->pdata); forw++)
      if(isalpha(data->pdata[forw]))
        {
          xtime=tolower(data->pdata[forw])-97;
          xtime=tolower(data->plug[xtime]);
          if(isupper(data->pdata[forw]))
            data->pdata[forw]=xtime-32;
          else
            data->pdata[forw]=xtime;
        }

  if (dsmash)
    {
      do {
        data->pdata=frq(data->pdata);
      } while(--xtime>0);
      printf("%s", data->pdata);
    }
  else if(data->hdata!=0)
   {
    printf("%lu\n", data->hdata);
   }
  else
    printf("%s", data->pdata);

  exit(0);

}

